function [fitness_lookup,order] = convex_hull_search_brute_force (in_pathway, outside_pathway, doplot)
%This function builds and explores a search tree looking for the optimal
%convex_hull which contains the points from the pathway and excludes those
%outside it.  It's called brute force, cos it starts with all the data
%included, and then explores all possible options with n-1 points, n-2
%points etc down to 3 points, pruning the tree as possible to ignore
%removing points which are not in the hull above it.

%fitness lookup is a 3 by n cell structure, the first column of the cell is
%a 1*m vector giving the defining vertices of that convex hull
%the second column is the fitness (p-value for that hull)
%the third column is the 2*p list of points outside the hull - this defines
%the branches of the tree which need to be searched below. 

%Setup the cell array for putting the fitness values in
fitness_lookup={[], [], []};

%Setup the values of the invariant variables for the cum_hyp_geom_dist
%function call
num_pathway=size(in_pathway,2);
num_total=num_pathway+size(outside_pathway,2);

%Calculate the starting convex hull (with everything)
full_hull= convhull(in_pathway(1,:),in_pathway(2,:));

% figure 
%     plot(outside_pathway(1,:),outside_pathway(2,:),'k*');
%     hold on
%     plot(in_pathway(1,:),in_pathway(2,:),'r*');
%     a = size(full_hull);
%     for i = 1:(a-1) 
%         x = [in_pathway(1,full_hull(i)),in_pathway(1,full_hull(i+1))]; 
%         y = [in_pathway(2,full_hull(i)),in_pathway(2,full_hull(i+1))]; 
%         plot(x,y,'r');
%     end
%     hold off

%Setup the current hull and the pathway index logical lookup for the things
%contained in this hull
current_hull=full_hull;
current_base=ones(1,num_pathway);

%Setup the flag that tells us if we need to continue the loop
cont=1;

%Setup the search tree - contains the list of branches waiting to be
%examined
look_at=[];
count=1;
%Now the slowest part of the code was the part where we checked through the
%fitness list in a for loop to see if we've seen a set before, so now treat
%the 1,0's in base as binary and make an array which is set to 1 when a thing is
%seen, then we can do direct lookups.
%seen_before = zeros (2^num_pathway, 1);
%Now changed so that it uses a list of hashed numbers rather than the
%binary lookup.
seen_before=[];
while cont==1
    current_hull =  current_hull(2:end);
    %Setup the list of points from the pathway outside the hull
    outside_hull=[];

    %Find all the points in the pathway outside the current hull
   %Matlab has a nicer way of doing this - using inpolygon I can do it for
   %all points simultaneously.
   inout=inpolygon(in_pathway(1,:),in_pathway(2,:), in_pathway(1,current_hull), in_pathway(2,current_hull));
   %So find gives the indices of the non-zero elements in an array, but I
   %want the zeros, so use not to switch them.
   outside_hull=find(not(inout));
   
   
    %Calculate the number inside the hull in the pathway
   num_sel_in_pway=sum(inout); 
   
   
    %check the non-pathway points to see if they're inside the hull and
    %increase the counter

  %Again do this the smarter way using inpolygon
  inout2=inpolygon(outside_pathway(1,:),outside_pathway(2,:), in_pathway(1,current_hull), in_pathway(2,current_hull));
  num_sel=sum(inout2)+num_sel_in_pway;
  
    %calculate the fitness of the current hull  
        %Let's try the built in function, which will at least get rid of
        %all the warnings, and hopefully might be quicker?
%     p = num_sel/num_total;
%     fitness=binocdf(num_sel_in_pway-1,num_pathway,p, 'upper');
    fitness = hygecdf(num_sel_in_pway-1, num_total, num_sel, num_pathway, 'upper');
    %The builtin function gives different values to my own code, whereas
    %the log hypergeometric function I found online gives the same results
    % Worked this out - this is cos the cdf in the built in function
    % assumes that you want the area from x+1 upwards ie. not including
    % the p(you pick x), whereas I want p(x or more).  
    
    
    %save the current hull along with fitness
    fitness_lookup{count,1} = current_hull;
    fitness_lookup{count,2}= fitness; 
    fitness_lookup{count,3} = outside_hull;
    fitness_lookup{count,4} = num_sel;
    fitness_lookup{count,5} = num_total;
    fitness_lookup{count,6} = num_pathway;
    fitness_lookup{count,7} = num_sel_in_pway;
    fitness_lookup{count,8} = in_pathway(:,current_hull);
    fitness_lookup{count,9} = current_base;
    % fitness_lookup{count,4} = current_base;
    %seen_before(hash(current_base))=1;

    count=count+1;
    
    %setup base counter for how many branches we have waiting to be looked
    %at
    length_la=length(look_at);
    %Keep count of the number of things added to look_at, so that we
    %can add without leaving empty slots.
    added=1;
    %Generate the list of branches which need looking at, by considering
    %the current set minus every point on the edge of the current hull
    for i=1:length(current_hull)
        temp_base=current_base;
        temp_base(current_hull(i))=0;
        
        %Only add the new set to look at if it has more than 2 points
        if (sum(temp_base)>2)
            
            %Only add to new set if it's not already been looked at.
            already_seen=0;
            h=hash(temp_base);
           % if (seen_before(h)==1)
           if (ismember(h,seen_before))
                already_seen=1;

            else
 
                look_at{added+length_la}=temp_base;
                added=added+1;
                seen_before=[seen_before,h];
                %seen_before(h)=1;
           end
            
        end
    end
    %stopping criteria
    if (isempty(look_at))
        cont=0;
    else    
        %Start looking at the first thing in the list
        current_base=look_at{1};
        look_at=look_at(2:end);

        %Get the convex hull, but this will be in the tempoarary index system
        %from the current base, we then need to translate this back into the
        %real index system
        current_hull_temp=convhull(in_pathway(1,current_base==1),in_pathway(2,current_base==1));
        cum_sum_base=ones(1,size(current_base,2));
        for i=1:size(current_base,2)
            %Sum up the ones and multiple it by the current number to keep the
            %zeros
            cum_sum_base(1,i)=sum(current_base(1,1:i))*current_base(1,i);
        end
        %Now I'm using inpolygon I need to preserve the order of the
        %current_hull_temp, otherwise that lookup doesn't work as the right
        %polygon doesn't get made.
      %So I need the index where each number from current_hull_temp is
      %found in cum_sum_base.
      current_hull=[];
      for i=1:size(current_hull_temp)
           current_hull(i)=find(cum_sum_base==current_hull_temp(i));
      end
    end
%     figure 
%          plot(outside_pathway(1,:),outside_pathway(2,:),'k*');
%          hold on
%          plot(in_pathway(1,:),in_pathway(2,:),'r*'); 
%          [a,b] = size(current_hull); 
%          for i = 1:(b-1) 
%              x = [in_pathway(1,current_hull(1,i)),in_pathway(1,current_hull(1,i+1))]; 
%              y = [in_pathway(2,current_hull(1,i)),in_pathway(2,current_hull(1,i+1))]; 
%              plot(x,y,'r'); 
%          end 
%          hold off
    
    
end
[~, best_index]=min(cell2mat(fitness_lookup(:,2)));
[sorted,order] = sort(cell2mat(fitness_lookup(:,2)));
if (doplot==1)
    figure
    %recalculate the convex hull to get the points in the right order - NOT
    %NEEDED now as points already in right order.
%    hull=[in_pathway(1,fitness_lookup{best_index,1}); in_pathway(2,fitness_lookup{best_index,1})];
%    K=convhull(hull(1,:),hull(2,:));
    fill(current_hull(1,K),current_hull(2,K),'r');
    hold on
    alpha(0.5);
    plot(outside_pathway(1,:),outside_pathway(2,:),'xb');
    plot(in_pathway(1,:),in_pathway(2,:),'ob');
    hold off
end
    
    
    


      