function best_list = genesetpca(data,entrez_gene_ids,algorithm,datatype, progressBar, genesetfile, whichComponents, searchAlg)
%Takes a table of data (each row one gene) and a list of entrez-gene ids (ordered as per data
%table) and does a pca of the data, and then outputs a table and calls the
%perl program to do pathway enrichment on the PCA loadings for each component 
%then finds the optimally over-represented convex hull in the pca loadings
%plot and outputs the plots.
%what_sig should be 'both' or 'one' and signifies if we should only report
%pathways significantly enriched in both PCs, or if just one PC is enough.
% It returns a cell of dimensions m x 1It returns a cell of dimensions m x 10 where m is the number of significant sets returned.
% The ten columns of the cell array contain the following;
% 1.	The pathway names (sorted alphabetically)
% 2.	The fdr adjusted p-value of the polygon
% 3.	The number of genes covered by the polygon
% 4.	The number of genes in the model in total (the same for every row)
% 5.	The number of genes in the set
% 6.	The number of genes in the set and covered by the polygon
% 7.	The positions of the corners of the polygon in the plot
% 8.	The entrez-gene ids for all the genes in the set
% 9.	The fdr adjusted p-value of the enrichment in the first principal component
% 10.	The fdr adjusted p-value of the enrichment in the second principal component
% 11.   The fdr adjusted p-value from the enrichment of the principal
% angles.

%Also read flat file of the the pathways which I downloaded earlier from
%CPDB website 
%Also did find/replace on commas for spaces, as the commas were being
%treated as a delimiter even though I specified that it should only use the
%tab.

pathways=readtable(genesetfile,'Delimiter','\t','ReadVariableNames',true);
%This determines how many pathways from the list we look at.  If it isn't
%size(pathways,1) then the first n pathways are examined, mostly for
%debugging to speed up the process.
num_pathways=size(pathways,1);
%Now setup some lookuplists so that I can quickly find all the genes in any
%given geneset.  
num_genes=size(entrez_gene_ids,1);
lookuplists=zeros(num_genes, num_pathways);
%cellfind = @(string)(@(cell_contents)(strcmp(string,cell_contents)));
waitbar(0, progressBar,'Creating lookup lists for genesets - step 2/4');       
for i=1:num_pathways
    pway=strsplit(char(pathways{i,4}),',');
%     for j=1:size(pway,2)
%         lookuplists(:,i)=lookuplists(:,i)+cellfun(cellfind(pway(j)),entrez_gene_ids);
%     end
    lookuplists(:,i)=ismember(entrez_gene_ids,pway);
    waitbar(i/num_pathways, progressBar,'Creating lookup lists for genesets - step 2/4');
end 
waitbar(0, progressBar,'Sorting PCA model');
switch datatype
    case 'DatasetCol'
        %first mean centre the data 
        data=table2array(data);
        data = data-repmat(mean(data),size(data,1),1);

        %next make pca model use just two components for now
        [~, all_loadings, ~] = pca(data,'NumComponents',max(whichComponents));
        pca_loadings=all_loadings(:,whichComponents);
    case 'DatasetRows'
        
        %Transpose the data, then do the same as for the columns
        data=table2array(data);
        data=data';
        %first mean centre the data 
        data = data-repmat(mean(data),size(data,1),1);

        %next make pca model use just two components for now
        [~, all_loadings, ~] = pca(data,'NumComponents',max(whichComponents));
        pca_loadings=all_loadings(:,whichComponents);

        
    case 'PreCalcLoadings'
        
        all_components=table2array(data);
        pca_loadings=all_components(:,whichComponents);

end
 %Calculate the principal angles
principal_angles=atan(pca_loadings(:,2)./pca_loadings(:,1));
%Problem is that when using these for enrichment you have to cut the circle
%somewhere, and whereever you cut the circle you risk loosing significant
%sets - so as a hackaround, have two cuts opposite each other, and take the
%minimum one of the two.
principal_angles2=principal_angles+pi;
principal_angles2(principal_angles2>pi)=principal_angles2(principal_angles2>pi)-pi;

waitbar(0, progressBar,'Calculating geneset enrichment - step 3/4');

switch algorithm
    case 'None'
        %If we aren't doing any enrichment calculations, then populate the
        %results with 1's for all pathways.
        pvalues_pc_enrichment=ones(num_pathways,3);
    case 'KSSelect'

        %Set up an array for the p-values
        pvalues_pc_enrichment=ones(num_pathways,3);
        
        %Now do the KS tests.
        for i=1:num_pathways
            if sum(lookuplists(:,i)==1)>2
                [~, pvalues_pc_enrichment(i,1), ~]=kstest2(pca_loadings(:,1), pca_loadings(lookuplists(:,i)==1,1));
                
                [~, pvalues_pc_enrichment(i,2), ~]=kstest2(pca_loadings(:,2), pca_loadings(lookuplists(:,i)==1,2));
                %Calculate the enrichment for both of these and see which
                %is the minimum one.
                [~, pvals_temp1(i), ~]=kstest2(principal_angles, principal_angles(lookuplists(:,i)==1));
                [~, pvals_temp2(i), ~]=kstest2(principal_angles2, principal_angles2(lookuplists(:,i)==1));
                pvalues_pc_enrichment(i,3)=min([pvals_temp1(i),pvals_temp2(i)],[],2);
            end
            waitbar(i/num_pathways, progressBar,'Calculating geneset enrichment - step 3/4');
        end
    case 'UGSEASelect'
        
        %get ranked gene list for each PC
        [pc1Values,sortIndexPC1]=sort(pca_loadings(:,1));
        RankedGeneList1=entrez_gene_ids(sortIndexPC1);
        [pc2Values,sortIndexPC2]=sort(pca_loadings(:,2));
        RankedGeneList2=entrez_gene_ids(sortIndexPC2);
        [PCAngles,sortIndexPCAngles]=sort(principal_angles);
        RankedGeneList3=entrez_gene_ids(sortIndexPCAngles);
        [PCAngles2,sortIndexPCAngles2]=sort(principal_angles2);
        RankedGeneList4=entrez_gene_ids(sortIndexPCAngles2);
        
        %Set up an array for the p-values
        pvalues_pc_enrichment=ones(num_pathways,3);
        waitbar(0.2, progressBar,'Calculating geneset enrichment - step 3/4');
        
        %Calculate GSEA enrichment for all pathways
        %then do the step below of finding the convex hulls

        pvalues_pc_enrichment(:,1)=mygsea_v2(pathways(1:num_pathways,:), RankedGeneList1, pc1Values, lookuplists);
        waitbar(0.4, progressBar,'Calculating geneset enrichment - step 3/4');
        pvalues_pc_enrichment(:,2)=mygsea_v2(pathways(1:num_pathways,:), RankedGeneList2, pc2Values, lookuplists);
        waitbar(0.6, progressBar,'Calculating geneset enrichment - step 3/4');
        pvals_temp1=mygsea_v2(pathways(1:num_pathways,:), RankedGeneList3, PCAngles, lookuplists);
        waitbar(0.8, progressBar,'Calculating geneset enrichment - step 3/4');
        pvals_temp2=mygsea_v2(pathways(1:num_pathways,:), RankedGeneList4, PCAngles2, lookuplists);
        pvalues_pc_enrichment(:,3)=min([pvals_temp1,pvals_temp2],[],2);
        waitbar(1, progressBar,'Calculating geneset enrichment - step 3/4');
end
%Find convex hull for each pathway
        for index_pathways=1:num_pathways
                           

            waitbar(index_pathways/num_pathways, progressBar,'Calculating convex hulls - step 4/4');       
            %Check pathway has at least 3 genes!
            if sum(lookuplists(:,index_pathways)==1)>2
                           
                %Perform the search for the best convex hull for the pathways
                if strcmp(searchAlg,'BruteForceBU')
                    convhull_results{index_pathways}=convex_hull_search_brute_force(pca_loadings(lookuplists(:,index_pathways)==1,:)', pca_loadings(lookuplists(:,index_pathways)==0,:)',0);
                elseif strcmp(searchAlg,'BruteForceTD')
                    convhull_results{index_pathways}=convex_hull_search_backwards(pca_loadings(lookuplists(:,index_pathways)==1,:)', pca_loadings(lookuplists(:,index_pathways)==0,:)',0); 
                elseif strcmp(searchAlg,'GreedyDensity')
                    convhull_results{index_pathways}=convex_hull_search_greedydensity(pca_loadings(lookuplists(:,index_pathways)==1,:)', pca_loadings(lookuplists(:,index_pathways)==0,:)',0); 
                else 
                    warning('Unknown Search Algorithm');
                    searchAlg
                end
            else 
                convhull_results{index_pathways}={};
                
            end
            
end
%Get best convex hull per pathway

%need a separate counter here, as some pathways have an empty list (they
%were too small for instance)
k=1;
for j=1:num_pathways
    if ~isempty(convhull_results{j})
        [~,besti]=min(cell2mat(convhull_results{j}(:,2)));
        best_list.genesetNames{k}=pathways{j,1};
        best_list.genesetPvalues(k)=cell2mat(convhull_results{j}(besti,2));
        best_list.polygonCovers(k)=cell2mat(convhull_results{j}(besti,4));
        best_list.genesetSize(k)=cell2mat(convhull_results{j}(besti,6));
        best_list.genesetCovered(k)=cell2mat(convhull_results{j}(besti,7));
        best_list.polygonCorners{k}=cell2mat(convhull_results{j}(besti,8));
        best_list.component1Enrichment(k)=pvalues_pc_enrichment(j,1);
        best_list.component2Enrichment(k)=pvalues_pc_enrichment(j,2);
        best_list.principalAngleEnrichment(k)=pvalues_pc_enrichment(j,3);
        best_list.genesetLookup(k,:)=lookuplists(j,:);
        best_list.geneIds{k}=entrez_gene_ids{lookuplists(:,j)==1};
        k=k+1;
    end;
    
end; 
%Add on the things which need to be saved which aren't specific per
%pathway.
best_list.totalGenes=num_genes;
best_list.loadings=pca_loadings;


 



